# coding: utf-8

"""
    Google Sheets API

    Reads and writes Google Sheets.

    The version of the OpenAPI document: v4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from google_sheets_ti.models.data_execution_status import DataExecutionStatus
from google_sheets_ti.models.data_source_column_reference import DataSourceColumnReference
from google_sheets_ti.models.filter_spec import FilterSpec
from google_sheets_ti.models.sort_spec import SortSpec
from typing import Optional, Set
from typing_extensions import Self

class DataSourceTable(BaseModel):
    """
    A data source table, which allows the user to import a static table of data from the DataSource into Sheets. This is also known as \"Extract\" in the Sheets editor.
    """ # noqa: E501
    column_selection_type: Optional[StrictStr] = Field(default=None, description="The type to select columns for the data source table. Defaults to SELECTED.", alias="columnSelectionType")
    columns: Optional[List[DataSourceColumnReference]] = Field(default=None, description="Columns selected for the data source table. The column_selection_type must be SELECTED.")
    data_execution_status: Optional[DataExecutionStatus] = Field(default=None, alias="dataExecutionStatus")
    data_source_id: Optional[StrictStr] = Field(default=None, description="The ID of the data source the data source table is associated with.", alias="dataSourceId")
    filter_specs: Optional[List[FilterSpec]] = Field(default=None, description="Filter specifications in the data source table.", alias="filterSpecs")
    row_limit: Optional[StrictInt] = Field(default=None, description="The limit of rows to return. If not set, a default limit is applied. Please refer to the Sheets editor for the default and max limit.", alias="rowLimit")
    sort_specs: Optional[List[SortSpec]] = Field(default=None, description="Sort specifications in the data source table. The result of the data source table is sorted based on the sort specifications in order.", alias="sortSpecs")
    __properties: ClassVar[List[str]] = ["columnSelectionType", "columns", "dataExecutionStatus", "dataSourceId", "filterSpecs", "rowLimit", "sortSpecs"]

    @field_validator('column_selection_type')
    def column_selection_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['DATA_SOURCE_TABLE_COLUMN_SELECTION_TYPE_UNSPECIFIED', 'SELECTED', 'SYNC_ALL']):
            raise ValueError("must be one of enum values ('DATA_SOURCE_TABLE_COLUMN_SELECTION_TYPE_UNSPECIFIED', 'SELECTED', 'SYNC_ALL')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DataSourceTable from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item_columns in self.columns:
                if _item_columns:
                    _items.append(_item_columns.to_dict())
            _dict['columns'] = _items
        # override the default output from pydantic by calling `to_dict()` of data_execution_status
        if self.data_execution_status:
            _dict['dataExecutionStatus'] = self.data_execution_status.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in filter_specs (list)
        _items = []
        if self.filter_specs:
            for _item_filter_specs in self.filter_specs:
                if _item_filter_specs:
                    _items.append(_item_filter_specs.to_dict())
            _dict['filterSpecs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sort_specs (list)
        _items = []
        if self.sort_specs:
            for _item_sort_specs in self.sort_specs:
                if _item_sort_specs:
                    _items.append(_item_sort_specs.to_dict())
            _dict['sortSpecs'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DataSourceTable from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "columnSelectionType": obj.get("columnSelectionType"),
            "columns": [DataSourceColumnReference.from_dict(_item) for _item in obj["columns"]] if obj.get("columns") is not None else None,
            "dataExecutionStatus": DataExecutionStatus.from_dict(obj["dataExecutionStatus"]) if obj.get("dataExecutionStatus") is not None else None,
            "dataSourceId": obj.get("dataSourceId"),
            "filterSpecs": [FilterSpec.from_dict(_item) for _item in obj["filterSpecs"]] if obj.get("filterSpecs") is not None else None,
            "rowLimit": obj.get("rowLimit"),
            "sortSpecs": [SortSpec.from_dict(_item) for _item in obj["sortSpecs"]] if obj.get("sortSpecs") is not None else None
        })
        return _obj


